'use strict';

var path = require('path');
var fs = require('fs');
var bn = require('bem-naming');
var logSymbols = require('log-symbols');

var defaultNaming = { elem: '-', elemDirPrefix: '', modDirPrefix: '_' };

module.exports = function (_ref) {
  var t = _ref.types;

  var namingOptions = void 0;
  var bemNaming = void 0;
  var bemLevels = void 0;
  var bemTechs = void 0;
  var getEntityFiles = void 0;
  var context = void 0;

  return {
    visitor: {
      Program: function Program(_, _ref2) {
        var _ref2$opts = _ref2.opts,
            naming = _ref2$opts.naming,
            levels = _ref2$opts.levels,
            techs = _ref2$opts.techs,
            cwd = _ref2$opts.cwd,
            filename = _ref2.file.opts.filename;

        namingOptions = Object.assign({}, defaultNaming, naming);
        bemNaming = bn(namingOptions);
        bemLevels = levels;
        bemTechs = techs;

        var rootEntity = path.basename(filename).split('.')[0];

        context = bemNaming.parse(rootEntity);

        getEntityFiles = function getEntityFiles(entity) {
          var prefixes = bemLevels.map(function (level) {
            return path.resolve(process.env.BEM_CWD || process.cwd(), path.join(level, entity.block, entity.elem ? '' + namingOptions.elemDirPrefix + entity.elem : '', entity.modName ? '' + namingOptions.modDirPrefix + entity.modName : '', bemNaming.stringify(entity)));
          });

          return bemTechs.reduce(function (res, tech) {
            return res.concat(prefixes.map(function (prefix) {
              return prefix + '.' + tech;
            }));
          }, []);
        };
      },
      ImportDeclaration: function ImportDeclaration(p, _ref3) {
        var filename = _ref3.file.opts.filename;

        if (p.node.source.value.match(/^(b|e|m)\:/)) {
          var localEntityName = p.node.specifiers[0] && p.node.specifiers[0].local.name;

          var importString = p.node.source.value;

          var requireIdx = null;

          var currentEntityRequires = parseEntityImport(importString, context).map(function (entity) {
            var possibleEntityFiles = getEntityFiles(entity);

            var requires = possibleEntityFiles.filter(fs.existsSync).map(function (entityFile, i) {
              !entity.modName && isFileJsModule(entityFile) && (requireIdx = i);

              var pa = path.relative(path.dirname(filename), entityFile);
              if (pa.charAt(0) !== '.') {
                pa = './' + pa;
              }
              return t.callExpression(t.identifier('require'), [t.stringLiteral(pa)]);
            });

            return { entity: entity, requires: requires };
          });

          var requires = currentEntityRequires.reduce(function (res, entity) {
            if (!entity.requires.length) {
              console.log(logSymbols.warning + '  Can\'t find declaration for BEM-entity \'' + bemNaming.stringify(entity.entity) + '\'. Maybe it has tech wich doesn\'t resolve in \'' + process.env.NODE_ENV + '\' env.');
            }

            return res.concat(entity.requires);
          }, []);

          var idx = requireIdx !== null;
          var requiresAst = t.arrayExpression(requires);
          var combinedAst = idx ? t.memberExpression(requiresAst, t.numericLiteral(requireIdx), true) : requiresAst;

          var replace = idx ? t.conditionalExpression(t.memberExpression(combinedAst, t.identifier('default')), t.callExpression(t.memberExpression(t.memberExpression(combinedAst, t.identifier('default')), t.identifier('applyDecls')), []), t.callExpression(t.memberExpression(combinedAst, t.identifier('applyDecls')), [])) : combinedAst;

          p.replaceWith(localEntityName ? t.variableDeclaration('const', [t.variableDeclarator(t.identifier(localEntityName), replace)]) : replace);
        }
      }
    }
  };
};

function parseEntityImport(entityImport, ctx) {
  var res = [];
  var main = {};

  entityImport.split(' ').forEach(function (importToken, i) {
    var split = importToken.split(':');
    var type = split[0];
    var tail = split[1];

    if (!i) {
      main.block = type === 'b' ? tail : ctx.block;
      type === 'e' && (main.elem = tail);
    } else if (type === 'e') {
      main.elem = tail;
    }

    (function () {
      switch (type) {
        case 'b':
        case 'e':
          res.length || res.push(main);
          break;

        case 'm':
          var splitMod = tail.split('=');
          var modName = splitMod[0];
          var modVals = splitMod[1];

          if (modVals) {
            modVals.split('|').forEach(function (modVal) {
              res.push(Object.assign({}, main, { modName: modName, modVal: modVal }));
            });
          } else {
            res.push(Object.assign({}, main, { modName: modName }));
          }
          break;
      }
    })();
  });

  return res;
}

function isFileJsModule(file) {
  return path.extname(file) === '.js';
};
